from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class EtcdClientCertAuth(BaseK8sContainerCheck):
    def __init__(self) -> None:
        # CIS-1.6 2.2
        id = "CKV_K8S_117"
        name = "Ensure that the --client-cert-auth argument is set to true"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        command = conf.get("command")
        if isinstance(command, list):
            if "etcd" in command and "--client-cert-auth=true" not in command:
                return CheckResult.FAILED

        return CheckResult.PASSED


check = EtcdClientCertAuth()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DynamodbGlobalTableRecovery(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Dynamodb point in time recovery (backup) is enabled for global tables"
        id = "CKV_AWS_165"
        supported_resources = ['aws_dynamodb_global_table']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        # This field cannot be set in terraform's aws_dyanmodb_global_table
        # https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/dynamodb_global_table
        return CheckResult.PASSED


check = DynamodbGlobalTableRecovery()



from __future__ import annotations

from pycep.typing import ResourceAttributes

from checkov.common.checks.base_check import BaseCheck
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.output.report import CheckType
from checkov.runner_filter import RunnerFilter


class Registry(BaseCheckRegistry):
    def __init__(self) -> None:
        self.check_id_to_enitity_map: dict[str, list[str]] = {}

        self.graph_registry = get_graph_checks_registry(CheckType.BICEP)
        self.graph_registry.load_checks()
        self.graph_check_ids = [check.id for check in self.graph_registry.checks]

        super().__init__()

    def register(self, check: BaseCheck) -> None:
        # a copy of the original method to be able to prioritize Bicep styled checks over the ARM equivalent
        if self._BaseCheckRegistry__loading_external_checks:  # type:ignore[attr-defined]  # they exist
            RunnerFilter.notify_external_check(check.id)

        # don't add an ARM check, if a Bicep graph check exists for it
        if check.id in self.graph_check_ids:
            return

        # remove the ARM check, if a Bicep check with the same check ID exists
        if check.id in self.check_id_to_enitity_map.keys():
            if check.__module__.split(".")[1] != "bicep":
                return

            entities = self.check_id_to_enitity_map[check.id]
            for entity in entities:
                checks = self.wildcard_checks if self._is_wildcard(entity) else self.checks
                check_idx = next((idx for idx, c in enumerate(checks[entity]) if c.id == check.id), None)
                if check_idx is not None:
                    del checks[entity][check_idx]

            del self.check_id_to_enitity_map[check.id]

        for entity in check.supported_entities:
            checks = self.wildcard_checks if self._is_wildcard(entity) else self.checks
            checks[entity].append(check)
            self.check_id_to_enitity_map.setdefault(check.id, []).append(entity)

        self._BaseCheckRegistry__all_registered_checks.append(check)  # type:ignore[attr-defined]  # they exist

    def extract_entity_details(self, entity: dict[str, ResourceAttributes]) -> tuple[str, str, ResourceAttributes]:  # type:ignore[override]  # it's ok
        resource_name, resource = next(iter(entity.items()))
        resource_type = resource["type"]
        return resource_type, resource_name, resource



from checkov.common.checks_infra.solvers.attribute_solvers.any_attribute_solver import AnyResourceSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.contains_attribute_solver import ContainsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_contains_attribute_solver import NotContainsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.ending_with_attribute_solver import EndingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.equals_attribute_solver import EqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.regex_match_attribute_solver import RegexMatchAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.exists_attribute_solver import ExistsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_ending_with_attribute_solver import NotEndingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_equals_attribute_solver import NotEqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_regex_match_attribute_solver import NotRegexMatchAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_exists_attribute_solver import NotExistsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_starting_with_attribute_solver import NotStartingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.starting_with_attribute_solver import StartingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.within_attribute_solver import WithinAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.greater_than_attribute_solver import GreaterThanAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.greater_than_or_equal_attribute_solver import GreaterThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.less_than_attribute_solver import LessThanAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.less_than_or_equal_attribute_solver import LessThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.jsonpath_equals_attribute_solver import JsonpathEqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.jsonpath_exists_attribute_solver import JsonpathExistsAttributeSolver  # noqa



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import Any

from checkov.bicep.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.models.consts import ANY_VALUE
from checkov.common.util.data_structures_utils import find_in_dict


class BaseResourceValueCheck(BaseResourceCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: "Iterable[CheckCategories]",
        supported_resources: "Iterable[str]",
        guideline: str | None = None,
        missing_block_result: CheckResult = CheckResult.FAILED,
    ) -> None:
        super().__init__(
            name=name, id=id, categories=categories, supported_resources=supported_resources, guideline=guideline
        )
        self.missing_block_result = missing_block_result

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        inspected_key = self.get_inspected_key()
        expected_values = self.get_expected_values()

        value = find_in_dict(conf, inspected_key)

        if value is None:
            return self.missing_block_result
        if ANY_VALUE in expected_values:
            return CheckResult.PASSED
        if value in expected_values:
            return CheckResult.PASSED
        # quite often string values are case-insensitive
        if isinstance(value, str) and value.lower() in [exp.lower() for exp in expected_values if isinstance(exp, str)]:
            return CheckResult.PASSED

        return self.missing_block_result

    @abstractmethod
    def get_inspected_key(self) -> str:
        """
        :return: JSONPath syntax path of the checked attribute
        """
        raise NotImplementedError()

    def get_expected_values(self) -> list[Any]:
        """
        Override the method with the list of acceptable values if the check has more than one possible expected value, given
        the inspected key
        :return: List of expected values, defaults to a list of the expected value
        """
        return [self.get_expected_value()]

    def get_expected_value(self) -> Any:
        """
        Returns the default expected value, governed by provider best practices
        """
        return True

    def get_evaluated_keys(self) -> list[str]:
        return [self.get_inspected_key()]



from checkov.common.models.enums import CheckResult

from checkov.gitlab_ci.checks.base_gitlab_ci_check import BaseGitlabCICheck
from checkov.yaml_doc.enums import BlockType


class AvoidDoublePipelines(BaseGitlabCICheck):
    def __init__(self):
        name = "Avoid creating rules that generate double pipelines"
        id = "CKV_GITLABCI_2"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=['*.rules']
        )

    def scan_entity_conf(self, conf):
        c = 0
        for x in conf:
            if "if" in x:
                value = x['if']
                if value.startswith('$CI_PIPELINE_SOURCE'):
                    c += 1
                if c > 1:
                    return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf
        

check = AvoidDoublePipelines()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class MQBrokerNotPubliclyExposed(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure MQ Broker is not publicly exposed"
        id = "CKV_AWS_69"
        supported_resources = ("aws_mq_broker",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        """
        Looks for public accessibility:
            https://www.terraform.io/docs/providers/aws/r/mq_broker.html#publicly_accessible
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-amazonmq-broker.html
        :param conf: aws_launch_configuration configuration
        :return: <CheckResult>
        """
        return "publicly_accessible"

    def get_forbidden_values(self) -> list[Any]:
        return [True]


check = MQBrokerNotPubliclyExposed()



from typing import Dict, Any

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class WAFACLCVE202144228(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure WAF prevents message lookup in Log4j2. See CVE-2021-44228 aka log4jshell"
        id = "CKV_AWS_192"
        supported_resources = ["AWS::WAFv2::WebACL"]
        categories = [CheckCategories.APPLICATION_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_keys = ["Properties/Rules"]
        properties = conf.get("Properties")
        if properties:
            rules = properties.get("Rules") or []
            for idx_rule, rule in enumerate(rules):
                self.evaluated_keys = [f"Properties/Rules/[{idx_rule}]/Statement"]
                statement = rule.get("Statement")
                if statement:
                    self.evaluated_keys = [f"Properties/Rules/[{idx_rule}]/Statement/ManagedRuleGroupStatement"]
                    managed_group = statement.get("ManagedRuleGroupStatement")
                    if managed_group:
                        self.evaluated_keys = [
                            f"Properties/Rules/[{idx_rule}]/Statement/ManagedRuleGroupStatement/Name"
                        ]
                        if managed_group.get("Name") == "AWSManagedRulesKnownBadInputsRuleSet":
                            self.evaluated_keys.append(
                                f"Properties/Rules/[{idx_rule}]/Statement/ManagedRuleGroupStatement/ExcludedRules"
                            )
                            excluded_rules = managed_group.get("ExcludedRules") or []
                            # rule 'Log4JRCE' should not be set to count
                            for idx_excluded_rule, excluded_rule in enumerate(excluded_rules):
                                if excluded_rule.get("Name") == "Log4JRCE":
                                    self.evaluated_keys = [
                                        f"Properties/Rules/[{idx_rule}]/Statement/ManagedRuleGroupStatement/Name",
                                        f"Properties/Rules/[{idx_rule}]/Statement/ManagedRuleGroupStatement/ExcludedRules/[{idx_excluded_rule}]/Name",
                                    ]
                                    return CheckResult.FAILED

                            self.evaluated_keys.append(f"Properties/Rules/[{idx_rule}]/OverrideAction/None")
                            override_action = rule.get("OverrideAction")
                            # check for group override
                            if override_action and next(iter(override_action.keys())) != "None":
                                return CheckResult.FAILED

                            return CheckResult.PASSED

        return CheckResult.FAILED


check = WAFACLCVE202144228()



import logging

from checkov.common.runners.runner_registry import RunnerRegistry

logger = logging.getLogger(__name__)


class RunnerDependencyHandler():
    """
    Scan runners for system dependencies, disable runners with failed deps
    """
    def __init__(self, runner_registry: RunnerRegistry) -> None:
        """
        RunnerDependencyHandler
        :param runner_registry: a populated runner registry
        """
        self.runner_registry = runner_registry

    def validate_runner_deps(self) -> None:
        """
        Checks if each runner declares any system dependancies by calling each runner's system_deps() function.
        This function can safley not exist, but if returns true, call check_system_deps() on the same function.
        The function would impliment it's own dependancy checks (see helm/runner.py for example).
        Sucessful check_system_deps() should return None, otherwise self.check_type to indicate a runner has failed deps.

        THen removes any runners with missing dependencies from runner_registry.
        """
        runners_with_unmatched_deps = []
        runner_names = []
        for runner in self.runner_registry.runners:
            system_deps = getattr(runner, 'system_deps', None)
            if system_deps:
                check_system_deps = getattr(runner, 'check_system_deps', None)
                if check_system_deps is not None:
                    result = check_system_deps()
                    if result is not None:
                        runner_names.append(result)
                        runners_with_unmatched_deps.append(runner)
            else:
                logging.debug(f"{runner.check_type}_runner declares no system dependency checks required.")
                continue
        
        if runners_with_unmatched_deps:
            logging.info(f"The following frameworks will automatically be disabled due to missing system dependencies: {','.join(runner_names)}")
            for runner in runners_with_unmatched_deps:
                self.runner_registry.remove_runner(runner)



from typing import List, Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.util.type_forcers import force_float


class GreaterThanAttributeSolver(BaseAttributeSolver):
    operator = Operators.GREATER_THAN

    def __init__(self, resource_types: List[str], attribute: Optional[str], value: Any) -> None:
        super().__init__(resource_types=resource_types, attribute=attribute, value=value)

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        vertex_attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None
        attr_float = force_float(vertex_attr)
        value_float = force_float(self.value)

        if vertex_attr is None:
            return False
        elif attr_float and value_float:
            return attr_float > value_float
        else:
            return str(vertex_attr) > str(self.value)



from dataclasses import dataclass
from typing import Optional


class Severity:
    def __init__(self, name: str, level: int) -> None:
        self.name = name
        self.level = level


@dataclass
class BcSeverities:
    NONE = 'NONE'
    LOW = 'LOW'
    MEDIUM = 'MEDIUM'
    HIGH = 'HIGH'
    CRITICAL = 'CRITICAL'
    MODERATE = 'MODERATE'
    IMPORTANT = 'IMPORTANT'


Severities = {
    BcSeverities.NONE: Severity(BcSeverities.NONE, 0),
    BcSeverities.LOW: Severity(BcSeverities.LOW, 1),
    BcSeverities.MEDIUM: Severity(BcSeverities.MEDIUM, 2),
    BcSeverities.MODERATE: Severity(BcSeverities.MEDIUM, 2),
    BcSeverities.HIGH: Severity(BcSeverities.HIGH, 3),
    BcSeverities.IMPORTANT: Severity(BcSeverities.HIGH, 3),
    BcSeverities.CRITICAL: Severity(BcSeverities.CRITICAL, 4),
}


def get_severity(severity: Optional[str]) -> Optional[Severity]:
    if not severity:
        return None
    return Severities.get(severity.upper())



from checkov.kubernetes.checks.resource.base_rbac_check import BaseRbacK8sCheck, RbacOperation


class RbacApproveCertificateSigningRequests(BaseRbacK8sCheck):
    def __init__(self):
        name = "Minimize ClusterRoles that grant permissions to approve CertificateSigningRequests"
        id = "CKV_K8S_156"
        supported_entities = ["ClusterRole"]
        super().__init__(name=name, id=id, supported_entities=supported_entities)

        # See https://kubernetes.io/docs/reference/access-authn-authz/certificate-signing-requests/
        self.failing_operations = [
            RbacOperation(
                apigroups=["certificates.k8s.io"],
                verbs=["update", "patch"],
                resources=["certificatesigningrequests/approval"]
            ),
            RbacOperation(
                apigroups=["certificates.k8s.io"],
                verbs=["approve"],
                resources=["signers"]
            ),
        ]


check = RbacApproveCertificateSigningRequests()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class InstanceBootVolumeIntransitEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure OCI Compute Instance boot volume has in-transit data encryption enabled"
        id = "CKV_OCI_4"
        supported_resources = ['oci_core_instance']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "launch_options/[0]/is_pv_encryption_in_transit_enabled"

    def get_expected_value(self):
        return True


check = InstanceBootVolumeIntransitEncryption()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_int

# https://docs.microsoft.com/en-us/azure/templates/microsoft.insights/2016-03-01/logprofiles


class MonitorLogProfileRetentionDays(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Activity Log Retention is set 365 days or greater"
        id = "CKV_AZURE_37"
        supported_resources = ("Microsoft.Insights/logprofiles",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "properties" in conf and "retentionPolicy" in conf["properties"]:
            retention = conf["properties"]["retentionPolicy"]
            if "enabled" in retention and str(retention["enabled"]).lower() == "true":
                if "days" in retention:
                    days = force_int(retention["days"])
                    if days is not None and (days == 0 or days >= 365):
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = MonitorLogProfileRetentionDays()



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class APIGatewayV2AccessLogging(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure API Gateway V2 has Access Logging enabled"
        id = "CKV_AWS_95"
        supported_resources = ['AWS::ApiGatewayV2::Stage', "AWS::Serverless::HttpApi"]
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/AccessLogSettings/DestinationArn'

    def get_expected_value(self):
        return ANY_VALUE


check = APIGatewayV2AccessLogging()
